/*
 * -------------------------------------------------------------------------
 * filter_int.c -- 
 * SWT - Scilab wavelet toolbox
 * Copyright (C) 2005-2006  Roger Liu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 */

#include "stack-c.h"
#include <stdio.h>
#include "swt.h"
#include "math.h"
#include "string.h"

int
intswtwrev (char *fname)
{
  static int l1, m1, n1, l2, m2, n2;
  static int minlhs = 1, maxlhs = 1, minrhs = 1, maxrhs = 1;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);

  if ((m1 == 1) && (n1 == 1))
    {
      sciprint ("Input should at least hold 2 elements!\n");
      return 0;
    }

  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Input must be vectors rather than matrixes!\n");
      return 0;
    }

  m2 = 1;
  n2 = n1 * m1;
  CreateVar (2, "d", &m2, &n2, &l2);
  wrev (stk (l1), n1 * m1, stk (l2));
  LhsVar (1) = 2;
  return 0;

}

int
intswtqmf (char *fname)
{
  static int l1, m1, n1, l2, m2, n2;
  static int minlhs = 1, maxlhs = 1, minrhs = 1, maxrhs = 1;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);

  if ((m1 == 1) && (n1 == 1))
    {
      sciprint ("Input should at least hold 2 elements!\n");
      return 0;
    }

  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Input must be vectors rather than matrixes!\n");
      return 0;
    }

  m2 = 1;
  n2 = n1 * m1;
  CreateVar (2, "d", &m2, &n2, &l2);
  qmf (stk (l1), n1 * m1, stk (l2));

  LhsVar (1) = 2;
  return 0;
}


int
intswtdown2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2;
  static int minlhs = 1, maxlhs = 1, minrhs = 1, maxrhs = 1;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);

  if ((m1 == 1) && (n1 == 1))
    {
      sciprint ("Input should at least hold 2 elements!\n");
      return 0;
    }
  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Input must be vectors rather than matrixes!\n");
      return 0;
    }

  m2 = 1;
  lenCal (n1 * m1, &n2);
  CreateVar (2, "d", &m2, &n2, &l2);
  downSamp2 (stk (l1), n1 * m1, stk (l2), n2);

  LhsVar (1) = 2;
  return 0;
}

int
intswtup2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2;
  static int minlhs = 1, maxlhs = 1, minrhs = 1, maxrhs = 1;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Input must be vectors rather than matrixes!\n");
      return 0;
    }

  m2 = 1;
  n2 = 2 * m1 * n1;
  CreateVar (2, "d", &m2, &n2, &l2);
  upSamp2 (stk (l1), n1 * m1, stk (l2));

  LhsVar (1) = 2;
  return;
}

int
intswtconv (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3;
  static int minlhs = 1, maxlhs = 1, minrhs = 2, maxrhs = 2;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);

  if (((m1 != 1) && (n1 != 1)) || ((m2 != 1) && (n2 != 1)))
    {
      sciprint ("Input must be vectors rather than matrixes!\n");
      return 0;
    }

  m3 = 1;
  n3 = m1 * n1 + m2 * n2 - 1;
  CreateVar (3, "d", &m3, &n3, &l3);
  swtconv (n1 * m1, stk (l1), n2 * m2, stk (l2), n3, stk (l3));
  LhsVar (1) = 3;
  return 0;
}


int
intswtwextend (char *fname)
{

  static int l1, m1, n1, l2, m2, n2, m3, n3, l3, l4, m4, n4;
  static int minlhs = 1, maxlhs = 1, minrhs = 2, maxrhs = 3;
  char c[] = { "symh" };
  int ffind;

  /* Check number of inputs (rhs=2) and outputs (lhs=1) */
  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);


  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  if ((m1 == 1) && (n1 == 1))
    {
      sciprint ("Input should be vectors!\n");
      return 0;
    }
  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Please use wextend2!\n");
      return 0;
    }
  if (m1 * n1 < istk (l2)[0])
    {
      sciprint ("input vector is too short to be extended!\n");
      return 0;
    }

  if (Rhs == 3)
    {
      GetRhsVar (3, "c", &m3, &n3, &l3);
      wextend_validate (cstk (l3), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extending method not available!\n");
	  return 0;
	}
      n4 = n1 * m1 + 2 * istk (l2)[0];
      m4 = 1;
      CreateVar (4, "d", &m4, &n4, &l4);
      wextend (stk (l1), n1 * m1, istk (l2)[0], cstk (l3), stk (l4));
      LhsVar (1) = 4;
    }
  else
    {
      m3 = 1;
      n3 = n1 * m1 + 2 * istk (l2)[0];
      CreateVar (3, "d", &m3, &n3, &l3);
      wextend (stk (l1), n1 * m1, istk (l2)[0], c, stk (l3));
      LhsVar (1) = 3;
    }
  return 0;
}


int
intswtwkeep (char *fname)
{

  static int l1, m1, n1, l2, m2, n2, l3, m3, n3;
  static int minlhs = 1, maxlhs = 1, minrhs = 2, maxrhs = 2;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);

  if ((m1 == 1) && (n1 == 1))
    {
      sciprint ("Input should be long enough!\n");
      return 0;
    }

  m3 = 1;
  n3 = istk (l2)[0];

  if (m1 * n1 < n3)
    {
      sciprint ("Please use a sensible length!\n");
      return 0;
    }

  CreateVar (3, "d", &m3, &n3, &l3);
  if ((m1 == 1) || (n1 == 1))
    wkeep (stk (l1), n1 * m1, stk (l3), n3);
  else
    {
      sciprint ("Please use wkeep2 instead!\n");
      return 0;
    }

  LhsVar (1) = 3;

  return 0;
}

int
intswtorthfilt (char *fname)
{

  static int l1, m1, n1, l2, m2, n2, l3, m3, n3;
  static int l4, m4, n4, l5, m5, n5;
  static int minlhs = 4, maxlhs = 4, minrhs = 1, maxrhs = 1;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  if ((m1 == 1) && (n1 == 1))
    {
      sciprint ("Input should be long enough!\n");
      return 0;
    }

  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Input should be vectors rather than matrixes!\n");
      return 0;
    }

  m2 = 1;
  m3 = 1;
  m4 = 1;
  m5 = 1;
  n2 = n1 * m1;
  n3 = n1 * m1;
  n4 = n1 * m1;
  n5 = n1 * m1;

  CreateVar (2, "d", &m2, &n2, &l2);
  CreateVar (3, "d", &m3, &n3, &l3);
  CreateVar (4, "d", &m4, &n4, &l4);
  CreateVar (5, "d", &m5, &n5, &l5);

  filgroup (stk (l1), n1 * m1, stk (l4), stk (l2), stk (l5), stk (l3));


  LhsVar (1) = 2;
  LhsVar (2) = 3;
  LhsVar (3) = 4;
  LhsVar (4) = 5;

  return 0;
}


int
intswtwave (char *fname)
{
  static int l1, m1, n1, l2, m2, n2;
  static int minlhs = 1, maxlhs = 1, minrhs = 1, maxrhs = 1;
  int find;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "c", &m1, &n1, &l1);
  wave_find (cstk (l1), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l1));
      return 0;
    }

  m2 = 1;
  wave_fil_len_cal (cstk (l1), &n2);

  CreateVar (2, "d", &m2, &n2, &l2);

  wave_fil_coef_extract (cstk (l1), stk (l2), n2);

  LhsVar (1) = 2;
  return 0;
}


int
intwmaxlev (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3;
  static int minlhs = 1, maxlhs = 1, minrhs = 2, maxrhs = 2;
  int find, stride, val, filterLen, stride1, stride2, val1, val2;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "c", &m2, &n2, &l2);
  wave_find (cstk (l2), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l2));
      return 0;
    }
  wave_fil_len_cal (cstk (l2), &filterLen);
  if ((m1 == 1) || (n1 == 1))
    {
      wave_len_validate (m1 * n1, filterLen, &stride, &val);
      if (val == 0)
	{
	  sciprint
	    ("The wavelet you select is not appropriate for that length of the vector!\n");
	  return 0;
	}
      else
	{
	  m3 = 1;
	  n3 = 1;
	  CreateVar (3, "i", &m3, &n3, &l3);
	  istk (l3)[0] = stride;
	  LhsVar (1) = 3;
	  return 0;
	}
    }
  else
    {
      wave_len_validate (m1, filterLen, &stride1, &val1);
      if (val1 == 0)
	sciprint
	  ("The wavelet you select is not appropriate for that row size of the matrix!\n");
      wave_len_validate (n1, filterLen, &stride2, &val2);
      if (val2 == 0)
	sciprint
	  ("The wavelet you select is not appropriate for that column size of the matrix!\n");
      if ((val1 == 0) || (val2 == 0))
	return 0;
      m3 = 1;
      n3 = 1;
      CreateVar (3, "i", &m3, &n3, &l3);
      istk (l3)[0] = (stride1 > stride2) ? stride2 : stride1;
      LhsVar (1) = 3;
      return 0;
    }
}

int
intswtdwt (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5;
  static int minlhs = 2, maxlhs = 2, minrhs = 2, maxrhs = 3;
  int filterLen, filterOutLength, downSampLen;
  char c[] = { "symh" };
  int find, stride, val, ffind;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);
  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "c", &m2, &n2, &l2);
  wave_find (cstk (l2), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l2));
      return 0;
    }
  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Please use dwt2 instead!\n");
      return 0;
    }

  /* wavelet filter length calculation */
  wave_fil_len_cal (cstk (l2), &filterLen);
  wave_len_validate (m1 * n1, filterLen, &stride, &val);

  if (val == 0)
    {
      sciprint ("Please use shorter wavelets!\n");
      return 0;
    }
  filterOutLength = filterLen + n1 * m1 - 1;
  lenCal (filterOutLength, &downSampLen);

  if (Rhs == 3)
    {
      GetRhsVar (3, "c", &m3, &n3, &l3);
      wextend_validate (cstk (l3), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m5 = 1;
      m4 = 1;
      n5 = downSampLen;
      n4 = downSampLen;
      CreateVar (4, "d", &m4, &n4, &l4);
      CreateVar (5, "d", &m5, &n5, &l5);
      dwtex (stk (l1), n1 * m1, cstk (l2), stk (l4), stk (l5),
	     filterLen, filterOutLength, downSampLen, cstk (l3));
      LhsVar (1) = 4;
      LhsVar (2) = 5;
    }
  else
    {
      m4 = 1;
      m3 = 1;
      n3 = downSampLen;
      n4 = downSampLen;
      CreateVar (3, "d", &m3, &n3, &l3);
      CreateVar (4, "d", &m4, &n4, &l4);
      dwtex (stk (l1), n1 * m1, cstk (l2), stk (l3), stk (l4),
	     filterLen, filterOutLength, downSampLen, c);
      LhsVar (1) = 3;
      LhsVar (2) = 4;
    }
  return 0;
}

int
intswtidwt (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6;
  static int minlhs = 1, maxlhs = 1, minrhs = 4, maxrhs = 5;
  int filterLen, filterOutLength;
  int find, ffind;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);
  GetRhsVar (4, "i", &m4, &n4, &l4);

  if (m1 * n1 != m2 * n2)
    {
      sciprint ("Input vectors are not approx and detail!\n");
      return 0;
    }
  if (((m1 != 1) && (n1 != 1)) || ((m2 != 1) && (n2 != 1)))
    {
      sciprint ("Please use idwt2 instead!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  /* wavelet filter length calculation */
  wave_fil_len_cal (cstk (l3), &filterLen);
  if (m1 * n1 < filterLen)
    {
      sciprint ("The wavelet is not appropriate for the input!\n");
      return 0;
    }
  filterOutLength = filterLen + 2 * n1 * m1 - 1;
  if (istk (l4)[0] > filterOutLength)
    {
      sciprint ("Result is shorter than you want!\n");
      return 0;
    }

  if (Rhs == 5)
    {
      GetRhsVar (5, "c", &m5, &n5, &l5);
      wextend_validate (cstk (l5), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m6 = 1;
      n6 = istk (l4)[0];
      CreateVar (6, "d", &m6, &n6, &l6);
      idwtex (cstk (l3), stk (l1), stk (l2), n1 * m1, filterLen,
	      filterOutLength, n6, stk (l6), cstk (l5));
      LhsVar (1) = 6;
      return 0;
    }
  else
    {
      m5 = 1;
      n5 = istk (l4)[0];
      CreateVar (5, "d", &m5, &n5, &l5);
      idwtex (cstk (l3), stk (l1), stk (l2), n1 * m1, filterLen,
	      filterOutLength, n5, stk (l5), c);
      LhsVar (1) = 5;
      return 0;
    }
}

int
intswtwavedec (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6;
  static int minlhs = 2, maxlhs = 2, minrhs = 3, maxrhs = 4;
  int calLen, temLen, count, total, filterLen;
  int find, val, stride, ffind;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);

  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Please use wavedec2 instead!\n");
      return 0;
    }
  if (istk (l2)[0] < 1)
    {
      sciprint ("Please use reasonable decomposition level!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  wave_fil_len_cal (cstk (l3), &filterLen);
  wave_len_validate (m1 * n1, filterLen, &stride, &val);

  if (val == 0)
    {
      sciprint ("Please use shorter wavelets!\n");
      return 0;
    }
  if (istk (l2)[0] > stride)
    {
      sciprint ("Please choose smaller decomposition level!\n");
      return 0;
    }

  if (Rhs == 4)
    {
      GetRhsVar (4, "c", &m4, &n4, &l4);
      wextend_validate (cstk (l4), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m6 = 1;
      m5 = 1;
      /* wavelet filter length calculation */

      n5 = 0;
      calLen = n1 * m1;
      for (count = 0; count < istk (l2)[0]; count++)
	{
	  calLen += filterLen - 1;
	  lenCal (calLen, &temLen);
	  n5 += temLen;
	  calLen = temLen;
	}
      n5 += temLen;
      n6 = istk (l2)[0] + 2;
      CreateVar (5, "d", &m5, &n5, &l5);
      CreateVar (6, "i", &m6, &n6, &l6);
      wave_dec_len_cal (cstk (l3), n1 * m1, istk (l2)[0], istk (l6), &total);
      wavedec (n1 * m1, stk (l1), n5, stk (l5), istk (l2)[0],
	       istk (l6), cstk (l3), cstk (l4));
      LhsVar (1) = 5;
      LhsVar (2) = 6;
    }
  else
    {
      m4 = 1;
      m5 = 1;
      n4 = 0;
      calLen = n1 * m1;
      for (count = 0; count < istk (l2)[0]; count++)
	{
	  calLen += filterLen - 1;
	  lenCal (calLen, &temLen);
	  n4 += temLen;
	  calLen = temLen;
	}
      n4 += temLen;
      n5 = istk (l2)[0] + 2;
      CreateVar (4, "d", &m4, &n4, &l4);
      CreateVar (5, "i", &m5, &n5, &l5);
      wave_dec_len_cal (cstk (l3), n1 * m1, istk (l2)[0], istk (l5), &total);
      wavedec (n1 * m1, stk (l1), n4, stk (l4), istk (l2)[0],
	       istk (l5), cstk (l3), c);
      LhsVar (1) = 4;
      LhsVar (2) = 5;

    }

  return 0;
}

int
intswtwaverec (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5;
  static int minlhs = 1, maxlhs = 1, minrhs = 3, maxrhs = 4;
  int find, ffind, len, count, val, filterLen;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);

  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("please input reasonable vectors!\n");
      return 0;
    }
  if ((m2 != 1) && (n2 != 1))
    {
      sciprint ("Please use waverec2 instead!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  len = 0;
  for (count = 0; count < (m2 * n2 - 1); count++)
    len += istk (l2)[count];
  if (len != m1 * n1)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  val = 0;
  for (count = 0; count < (m2 * n2 - 1); count++)
    {
      if (istk (l2)[count] > istk (l2)[count + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  wave_fil_len_cal (cstk (l3), &filterLen);
  if (istk (l2)[0] < filterLen)
    {
      sciprint ("The wavelet you selected is not valide for this coef!\n");
      return 0;
    }

  if (Rhs == 4)
    {
      GetRhsVar (4, "c", &m4, &n4, &l4);
      wextend_validate (cstk (l4), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m5 = 1;
      n5 = istk (l2)[n2 * m2 - 1];
      CreateVar (5, "d", &m5, &n5, &l5);
      waverec (n1 * m1, stk (l1), n5, stk (l5), (n2 * m2 - 2),
	       istk (l2), cstk (l3), cstk (l4));
      LhsVar (1) = 5;
    }
  else
    {
      m4 = 1;
      n4 = istk (l2)[n2 * m2 - 1];
      CreateVar (4, "d", &m4, &n4, &l4);
      waverec (n1 * m1, stk (l1), n4, stk (l4), (n2 * m2 - 2),
	       istk (l2), cstk (l3), c);
      LhsVar (1) = 4;
    }

  return 0;
}

int
intswtwrcoef (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6, l7, m7, n7;
  static int minlhs = 1, maxlhs = 1, minrhs = 5, maxrhs = 6;
  int find, ffind, val, len, count, filterLen;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "c", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);
  GetRhsVar (3, "i", &m3, &n3, &l3);
  GetRhsVar (4, "c", &m4, &n4, &l4);
  GetRhsVar (5, "i", &m5, &n5, &l5);

  if ((strcmp (cstk (l1), "a") != 0) && (strcmp (cstk (l1), "d") != 0))
    {
      sciprint ("Please indicate 'a' for approximate or 'd' for details!\n");
      return 0;
    }
  if ((m2 != 1) && (n2 != 1))
    {
      sciprint ("please input reasonable vectors!\n");
      return 0;
    }
  if ((m3 != 1) && (n3 != 1))
    {
      sciprint ("Please use waverec2 instead!\n");
      return 0;
    }
  wave_find (cstk (l4), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l4));
      return 0;
    }
  len = 0;
  for (count = 0; count < (m3 * n3 - 1); count++)
    len += istk (l3)[count];
  if (len != m2 * n2)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  val = 0;
  for (count = 0; count < (m3 * n3 - 1); count++)
    {
      if (istk (l3)[count] > istk (l3)[count + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  if ((istk (l5)[0] < 1) || (istk (l5)[0] > (m3 * n3 - 2)))
    {
      sciprint ("Please choose reasonable levels!\n");
      return 0;
    }
  wave_fil_len_cal (cstk (l4), &filterLen);
  if (istk (l3)[0] < filterLen)
    {
      sciprint ("The wavelet you selected is not valide for this coef!\n");
      return 0;
    }



  if (Rhs == 6)
    {
      GetRhsVar (6, "c", &m6, &n6, &l6);
      wextend_validate (cstk (l6), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m7 = 1;
      n7 = istk (l3)[n3 * m3 - 1];
      CreateVar (7, "d", &m7, &n7, &l7);
      wrcoef (stk (l2), n2 * m2, n3 * m3 - 2, cstk (l4), stk (l7), n7,
	      cstk (l1), istk (l5)[0], istk (l3), cstk (l6));
      LhsVar (1) = 7;
    }
  else
    {
      m6 = 1;
      n6 = istk (l3)[n3 * m3 - 1];
      CreateVar (6, "d", &m6, &n6, &l6);
      wrcoef (stk (l2), n2 * m2, n3 * m3 - 2, cstk (l4), stk (l6), n6,
	      cstk (l1), istk (l5)[0], istk (l3), c);
      LhsVar (1) = 6;
    }
  return 0;
}

int
intswtdetcoef (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int minlhs = 1, maxlhs = 1, minrhs = 3, maxrhs = 3;
  int len, val, count;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "i", &m3, &n3, &l3);

  len = 0;
  for (count = 0; count < (m2 * n2 - 1); count++)
    len += istk (l2)[count];
  if (len != m1 * n1)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  val = 0;
  for (count = 0; count < (m2 * n2 - 1); count++)
    {
      if (istk (l2)[count] > istk (l2)[count + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  if ((istk (l3)[0] < 1) || (istk (l3)[0] > (m2 * n2 - 2)))
    {
      sciprint ("Please choose reasonable levels!\n");
      return 0;
    }
  if ((m2 != 1) && (n2 != 1))
    {
      sciprint ("Please use detcoef2 instead!\n");
      return 0;
    }
  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }


  m4 = 1;
  n4 = istk (l2)[n2 * m2 - istk (l3)[0] - 1];
  CreateVar (4, "d", &m4, &n4, &l4);
  detcoef (n1 * m1, stk (l1), n4, stk (l4), (n2 * m2 - 2), istk (l3)[0],
	   istk (l2));
  LhsVar (1) = 4;
  return 0;
}

int
intswtappcoef (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6;
  static int minlhs = 1, maxlhs = 1, minrhs = 4, maxrhs = 5;
  char c[] = { "symh" };
  int len, val, count, find, ffind, filterLen;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);
  GetRhsVar (4, "i", &m4, &n4, &l4);

  len = 0;
  for (count = 0; count < (m2 * n2 - 1); count++)
    len += istk (l2)[count];
  if (len != m1 * n1)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  val = 0;
  for (count = 0; count < (m2 * n2 - 1); count++)
    {
      if (istk (l2)[count] > istk (l2)[count + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  if ((istk (l4)[0] < 1) || (istk (l4)[0] > (m2 * n2 - 2)))
    {
      sciprint ("Please choose reasonable levels!\n");
      return 0;
    }
  if ((m2 != 1) && (n2 != 1))
    {
      sciprint ("Please use appcoef2 instead!\n");
      return 0;
    }
  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Inputs are not coef and length array!\n");
      return 0;
    }
  wave_fil_len_cal (cstk (l3), &filterLen);
  if (istk (l2)[0] < filterLen)
    {
      sciprint ("The wavelet you selected is not valide for this coef!\n");
      return 0;
    }


  if (Rhs == 5)
    {
      GetRhsVar (5, "c", &m5, &n5, &l5);
      wextend_validate (cstk (l5), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m6 = 1;
      n6 = istk (l2)[n2 * m2 - 2 - istk (l4)[0] + 1];
      CreateVar (6, "d", &m6, &n6, &l6);
      appcoef (stk (l1), n1 * m1, (n2 * m2 - 2), cstk (l3), stk (l6),
	       n6, istk (l4)[0], istk (l2), cstk (l5));
      LhsVar (1) = 6;
    }
  else
    {
      m5 = 1;
      n5 = istk (l2)[n2 * m2 - 2 - istk (l4)[0] + 1];
      CreateVar (5, "d", &m5, &n5, &l5);
      appcoef (stk (l1), n1 * m1, (n2 * m2 - 2), cstk (l3), stk (l5),
	       n5, istk (l4)[0], istk (l2), c);
      LhsVar (1) = 5;
    }

  return 0;
}

int
intswtdwt2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6, l7, m7, n7;
  static int minlhs = 4, maxlhs = 4, minrhs = 2, maxrhs = 3;
  int filterLen, filterOutLength;
  int find, val1, val2, ffind, stride1, stride2;
  char c[] = { "symh" };


  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "c", &m2, &n2, &l2);

  if ((m1 == 1) || (n1 == 1))
    {
      sciprint ("Please input image matrixes rather than vectors!\n");
      return 0;
    }
  wave_find (cstk (l2), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l2));
      return 0;
    }
  wave_fil_len_cal (cstk (l2), &filterLen);
  wave_len_validate (m1, filterLen, &stride1, &val1);
  if (val1 == 0)
    sciprint
      ("The wavelet you select is not appropriate for that row size of the matrix!\n");
  wave_len_validate (n1, filterLen, &stride2, &val2);
  if (val2 == 0)
    sciprint
      ("The wavelet you select is not appropriate for that column size of the matrix!\n");
  if ((val1 == 0) || (val2 == 0))
    return 0;


  if (Rhs == 3)
    {
      GetRhsVar (3, "c", &m3, &n3, &l3);
      wextend_validate (cstk (l3), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      filterOutLength = filterLen + m1 - 1;
      lenCal (filterOutLength, &m4);
      filterOutLength = filterLen + n1 - 1;
      lenCal (filterOutLength, &n4);
      m5 = m4;
      m6 = m4;
      m7 = m4;
      n5 = n4;
      n6 = n4;
      n7 = n4;
      CreateVar (4, "d", &m4, &n4, &l4);
      CreateVar (5, "d", &m5, &n5, &l5);
      CreateVar (6, "d", &m6, &n6, &l6);
      CreateVar (7, "d", &m7, &n7, &l7);
      swtdwt2Dex (stk (l1), m1, n1, stk (l4), stk (l5), stk (l6),
		  stk (l7), cstk (l2), cstk (l3));
      LhsVar (1) = 4;
      LhsVar (2) = 5;
      LhsVar (3) = 6;
      LhsVar (4) = 7;
    }
  else
    {
      filterOutLength = filterLen + m1 - 1;
      lenCal (filterOutLength, &m3);
      filterOutLength = filterLen + n1 - 1;
      lenCal (filterOutLength, &n3);
      m4 = m3;
      m5 = m3;
      m6 = m3;
      n4 = n3;
      n5 = n3;
      n6 = n3;
      CreateVar (3, "d", &m3, &n3, &l3);
      CreateVar (4, "d", &m4, &n4, &l4);
      CreateVar (5, "d", &m5, &n5, &l5);
      CreateVar (6, "d", &m6, &n6, &l6);
      swtdwt2Dex (stk (l1), m1, n1, stk (l3), stk (l4), stk (l5),
		  stk (l6), cstk (l2), c);
      LhsVar (1) = 3;
      LhsVar (2) = 4;
      LhsVar (3) = 5;
      LhsVar (4) = 6;
    }
  return 0;
}


int
intswtwkeep2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, n4, m4;
  static int minlhs = 1, maxlhs = 1, minrhs = 3, maxrhs = 3;
  /* int m, n; */

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "i", &m3, &n3, &l3);
  /* m=istk(l2)[0];
     n=istk(l3)[0];
   */

  if ((m1 == 1) || (n1 == 1))
    {
      sciprint ("Please wkeep instead becasue of vectors input!\n");
      return 0;
    }
  if (istk (l2)[0] > m1)
    {
      sciprint ("Please use wextend2 to do this!\n");
      /* sciprint("m1=%d, row=%d\n",m1,istk(l2)[0]); */
      return 0;
    }
  if ((istk (l3)[0]) > n1)
    {
      sciprint ("Please use wextend2 to do this!\n");
      /* sciprint("n1=%d, column=%d\n",n1,istk(l3)[0]); */
      return 0;
    }
  if ((istk (12)[0]) < 1)
    {
      sciprint ("Row size you want is not valid!\n");
      return 0;
    }
  if ((istk (l3)[0]) < 1)
    {
      sciprint ("Col size you want is not valid!\n");
      return 0;
    }

  m4 = istk (l2)[0];
  n4 = istk (l3)[0];
  CreateVar (4, "d", &m4, &n4, &l4);
  wkeep2 (stk (l1), m1, n1, stk (l4), m4, n4);
  LhsVar (1) = 4;
  return 0;
}

int
intswtwextend2 (char *fname)
{

  static int l1, m1, n1, l2, m2, n2, m3, n3, l3;
  static int l4, m4, n4, l5, m5, n5;
  static int minlhs = 1, maxlhs = 1, minrhs = 4, maxrhs = 4;
  int ffind;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "c", &m2, &n2, &l2);
  GetRhsVar (3, "i", &m3, &n3, &l3);
  GetRhsVar (4, "i", &m4, &n4, &l4);

  if ((m1 == 1) || (n1 == 1))
    {
      sciprint ("Please use wextend to do this!\n");
      return 0;
    }
  wextend_validate (cstk (l2), &ffind);
  if (ffind == 0)
    {
      sciprint ("Extend Method not available!\n");
      return 0;
    }
  if ((istk (l3)[0] < 1) || (istk (l4)[0] < 1))
    {
      sciprint ("Size to be extended is not valid!\n");
      return 0;
    }
  if ((istk (l3)[0] > m1) || (istk (l4)[0] > n1))
    {
      sciprint ("Size to be extended is not valid!\n");
      return 0;
    }

  m5 = m1 + 2 * istk (l3)[0];
  n5 = n1 + 2 * istk (l4)[0];
  CreateVar (5, "d", &m5, &n5, &l5);
  wextend2 (stk (l1), m1, n1, istk (l3)[0], istk (l4)[0],
	    cstk (l2), stk (l5));
  LhsVar (1) = 5;
  return 0;
}

int
intswtidwt2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6, l7, m7, n7, l8, m8, n8;
  static int l9, m9, n9;
  static int minlhs = 1, maxlhs = 1, minrhs = 7, maxrhs = 8;
  char c[] = { "symh" };
  int find, ffind, filterLen, filterOutRowLength, filterOutColLength;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);
  GetRhsVar (3, "d", &m3, &n3, &l3);
  GetRhsVar (4, "d", &m4, &n4, &l4);
  GetRhsVar (5, "c", &m5, &n5, &l5);
  GetRhsVar (6, "i", &m6, &n6, &l6);
  GetRhsVar (7, "i", &m7, &n7, &l7);

  if ((m1 != m2) || (m1 != m3) || (m1 != m4))
    {
      sciprint ("Inputs are not coefs!\n");
      return 0;
    }
  if ((n1 != n2) || (n1 != n3) || (n1 != n4))
    {
      sciprint ("Inputs are not coefs!\n");
      return 0;
    }
  if ((m1 == 1) || (n1 == 1))
    {
      sciprint ("Input should be matrixes!\n");
      return 0;
    }
  wave_find (cstk (l5), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l5));
      return 0;
    }
  if ((istk (l6)[0] < 1) || (istk (l7)[0] < 1))
    {
      sciprint ("matrix size you want is not valid!\n");
      return 0;
    }
  wave_fil_len_cal (cstk (l5), &filterLen);
  filterOutRowLength = filterLen + 2 * m1 - 1;
  filterOutColLength = filterLen + 2 * n1 - 1;
  if ((istk (l6)[0] > filterOutRowLength)
      || (istk (l7)[0] > filterOutColLength))
    {
      sciprint ("matrix size you want is not valid!\n");
      return 0;
    }
  if ((m1 < filterLen) || (n1 < filterLen))
    {
      sciprint
	("The wavelet you select is not valid for that size of the matrix!\n");
      return 0;
    }

  if (Rhs == 8)
    {
      GetRhsVar (8, "c", &m8, &n8, &l8);
      wextend_validate (cstk (l8), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m9 = istk (l6)[0];
      n9 = istk (l7)[0];
      CreateVar (9, "d", &m9, &n9, &l9);
      swtidwt2Dex (stk (l1), stk (l2), stk (l3), stk (l4), m1, n1,
		   stk (l9), m9, n9, cstk (l5), cstk (l8));
      LhsVar (1) = 9;
    }
  else
    {
      m8 = istk (l6)[0];
      n8 = istk (l7)[0];
      CreateVar (8, "d", &m8, &n8, &l8);
      swtidwt2Dex (stk (l1), stk (l2), stk (l3), stk (l4), m1, n1,
		   stk (l8), m8, n8, cstk (l5), c);
      LhsVar (1) = 8;
    }
  return 0;
}

int
intswtwavedec2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6;
  static int minlhs = 2, maxlhs = 2, minrhs = 3, maxrhs = 4;
  int *pLen, total, row, col, ffind;
  int find, filterLen, stride1, stride2, val1, val2, stride;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);

  if ((m1 == 1) || (n1 == 1))
    {
      sciprint ("Please input matrixes or use wavedec instead!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  wave_fil_len_cal (cstk (l3), &filterLen);
  wave_len_validate (m1, filterLen, &stride1, &val1);
  if (val1 == 0)
    sciprint
      ("The wavelet you select is not appropriate for that row size of the matrix!\n");
  wave_len_validate (n1, filterLen, &stride2, &val2);
  if (val2 == 0)
    sciprint
      ("The wavelet you select is not appropriate for that column size of the matrix!\n");
  if ((val1 == 0) || (val2 == 0))
    return 0;
  stride = (stride1 > stride2) ? stride2 : stride1;
  if ((istk (l2)[0] < 1) || (istk (l2)[0] > stride))
    {
      sciprint ("Please input a reasonable level!\n");
      return 0;
    }

  pLen = malloc ((istk (l2)[0] + 1) * 2 * sizeof (int));
  matrix_wavedec_len_cal (m1, n1, istk (l2)[0], cstk (l3), pLen);
  wave_mem_cal (pLen, istk (l2)[0], &total);

  if (Rhs == 4)
    {
      GetRhsVar (4, "c", &m4, &n4, &l4);
      wextend_validate (cstk (l4), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m5 = 1;
      n5 = total;
      m6 = istk (l2)[0] + 1;
      n6 = 2;
      CreateVar (5, "d", &m5, &n5, &l5);
      CreateVar (6, "i", &m6, &n6, &l6);
      wavedec2 (stk (l1), istk (l2)[0], cstk (l3), pLen, stk (l5), cstk (l4));
      for (row = 0; row < m6; row++)
	{
	  for (col = 0; col < n6; col++)
	    istk (l6)[row + col * m6] = pLen[col + row * n6];
	}
      LhsVar (1) = 5;
      LhsVar (2) = 6;
    }
  else
    {
      m4 = 1;
      n4 = total;
      m5 = istk (l2)[0] + 1;
      n5 = 2;
      CreateVar (4, "d", &m4, &n4, &l4);
      CreateVar (5, "i", &m5, &n5, &l5);
      wavedec2 (stk (l1), istk (l2)[0], cstk (l3), pLen, stk (l4), c);
      for (row = 0; row < m5; row++)
	{
	  for (col = 0; col < n5; col++)
	    istk (l5)[row + col * m5] = pLen[col + row * n5];
	}
      LhsVar (1) = 4;
      LhsVar (2) = 5;
    }
  free (pLen);
  return 0;
}

int
intswtwaverec2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5;
  static int minlhs = 1, maxlhs = 1, minrhs = 3, maxrhs = 4;
  int *pLen, row, col, size;
  int find, ffind, val, filterLen;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);

  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Inputs are not coefs!\n");
      return 0;
    }
  if ((m2 == 1) || (n2 == 1))
    {
      sciprint ("Inputs are not size arrays!\n");
      return 0;
    }
  size = 0;
  for (row = 0; row < (m2 - 1); row++)
    {
      size += (istk (l2)[row]) * (istk (l2)[row + m2]);
    }
  size = size * 3 + (istk (l2)[0]) * (istk (l2)[m2]);
  if (m1 * n1 != size)
    {
      sciprint ("Inputs are not size array and coefs!\n");
      return 0;
    }
  val = 0;
  for (row = 0; row < (m2 - 1); row++)
    {
      if (istk (l2)[row] > istk (l2)[row + 1])
	val += 1;
      if (istk (l2)[row + m2] > istk (l2)[row + m2 + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not size array!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  wave_fil_len_cal (cstk (l3), &filterLen);
  if ((istk (l2)[0] < filterLen) || (istk (l2)[m2] < filterLen))
    {
      sciprint ("The wavelet you select is not valid!\n");
      return 0;
    }


  pLen = malloc (m2 * n2 * sizeof (int));
  for (row = 0; row < n2; row++)
    {
      for (col = 0; col < m2; col++)
	pLen[row + col * n2] = istk (l2)[col + row * m2];
    }
  if (Rhs == 4)
    {
      GetRhsVar (4, "c", &m4, &n4, &l4);
      wextend_validate (cstk (l4), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m5 = pLen[(m2 - 1) * n2];
      n5 = pLen[(m2 - 1) * n2 + 1];
      CreateVar (5, "d", &m5, &n5, &l5);
      waverec2 (stk (l1), pLen, (m2 - 1), cstk (l3), stk (l5), cstk (l4));
      LhsVar (1) = 5;
    }
  else
    {
      m4 = pLen[(m2 - 1) * n2];
      n4 = pLen[(m2 - 1) * n2 + 1];
      CreateVar (4, "d", &m4, &n4, &l4);
      waverec2 (stk (l1), pLen, (m2 - 1), cstk (l3), stk (l4), c);
      LhsVar (1) = 4;
    }

  free (pLen);
  return 0;
}

int
intswtdetcoef2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6, l7, m7, n7;
  static int minlhs = 1, maxlhs = 3, minrhs = 4, maxrhs = 4;
  int *pLen, row, col, val, size;
  static char h[] = { "h" };
  static char v[] = { "v" };
  static char d[] = { "d" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "c", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);
  GetRhsVar (3, "i", &m3, &n3, &l3);
  GetRhsVar (4, "i", &m4, &n4, &l4);

  val = 0;
  val +=
    strcmp (cstk (l1), h) + strcmp (cstk (l1), v) + strcmp (cstk (l1), d);
  val += strcmp (cstk (l1), "all");
  if (val == 0)
    {
      sciprint ("Please input the type you want!\n");
      return 0;
    }
  if ((m2 != 1) && (n2 != 1))
    {
      sciprint ("Inputs are not coefs!\n");
      return 0;
    }
  if ((m3 == 1) || (n3 == 1))
    {
      sciprint ("Inputs are not size arrays!\n");
      return 0;
    }
  size = 0;
  for (row = 0; row < (m3 - 1); row++)
    {
      size += (istk (l3)[row]) * (istk (l3)[row + m3]);
    }
  size = size * 3 + (istk (l3)[0]) * (istk (l3)[m3]);
  if (m2 * n2 != size)
    {
      sciprint ("Inputs are not size array and coefs!\n");
      return 0;
    }
  val = 0;
  for (row = 0; row < (m3 - 1); row++)
    {
      if (istk (l3)[row] > istk (l3)[row + 1])
	val += 1;
      if (istk (l3)[row + m3] > istk (l3)[row + m3 + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not size array!\n");
      return 0;
    }
  if ((istk (l4)[0] < 1) || (istk (l4)[0] > (m3 - 1)))
    {
      sciprint ("Please input a reasonable level!\n");
      return 0;
    }


  pLen = malloc (m3 * n3 * sizeof (int));
  for (row = 0; row < n3; row++)
    {
      for (col = 0; col < m3; col++)
	pLen[row + col * n3] = istk (l3)[col + row * m3];
    }

  if (strcmp (cstk (l1), "all") == 0)
    {
      m5 = pLen[(m3 - 1 - istk (l4)[0]) * n3];
      n5 = pLen[(m3 - 1 - istk (l4)[0]) * n3 + 1];
      m6 = m5;
      m7 = m5;
      n6 = n5;
      n7 = n5;
      CreateVar (5, "d", &m5, &n5, &l5);
      CreateVar (6, "d", &m6, &n6, &l6);
      CreateVar (7, "d", &m7, &n7, &l7);
      detcoef2 (stk (l2), pLen, (m3 - 1), istk (l4)[0], h, stk (l5));
      detcoef2 (stk (l2), pLen, (m3 - 1), istk (l4)[0], v, stk (l6));
      detcoef2 (stk (l2), pLen, (m3 - 1), istk (l4)[0], d, stk (l7));
      LhsVar (1) = 5;
      LhsVar (2) = 6;
      LhsVar (3) = 7;
    }
  else
    {
      m5 = pLen[(m3 - 1 - istk (l4)[0]) * n3];
      n5 = pLen[(m3 - 1 - istk (l4)[0]) * n3 + 1];
      CreateVar (5, "d", &m5, &n5, &l5);
      detcoef2 (stk (l2), pLen, (m3 - 1), istk (l4)[0], cstk (l1), stk (l5));
      LhsVar (1) = 5;
    }
  free (pLen);
  return 0;
}

int
intswtwrcoef2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6, l7, m7, n7;
  static int minlhs = 1, maxlhs = 1, minrhs = 5, maxrhs = 6;
  int *pLen, row, col, val, find, size, ffind, filterLen;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "c", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);
  GetRhsVar (3, "i", &m3, &n3, &l3);
  GetRhsVar (4, "c", &m4, &n4, &l4);
  GetRhsVar (5, "i", &m5, &n5, &l5);

  val = 0;
  if (strcmp (cstk (l1), "a") == 0)
    val = 1;
  else if (strcmp (cstk (l1), "h") == 0)
    val = 1;
  else if (strcmp (cstk (l1), "v") == 0)
    val = 1;
  else if (strcmp (cstk (l1), "d") == 0)
    val = 1;
  else
    val = 0;

  /* val += strcmp(cstk(l1),"a")+strcmp(cstk(l1),"d");
     val += strcmp(cstk(l1),"h")+strcmp(cstk(l1),"v");
   */
  if (val == 0)
    {
      sciprint ("Please input the type you want!\n");
      return 0;
    }
  if ((m2 != 1) && (n2 != 1))
    {
      sciprint ("Inputs are not coefs!\n");
      return 0;
    }
  if ((m3 == 1) || (n3 == 1))
    {
      sciprint ("Inputs are not size arrays!\n");
      return 0;
    }
  size = 0;
  for (row = 0; row < (m3 - 1); row++)
    {
      size += (istk (l3)[row]) * (istk (l3)[row + m3]);
    }
  size = size * 3 + (istk (l3)[0]) * (istk (l3)[m3]);
  if (m2 * n2 != size)
    {
      sciprint ("Inputs are not size array and coefs!\n");
      return 0;
    }
  val = 0;
  for (row = 0; row < (m3 - 1); row++)
    {
      if (istk (l3)[row] > istk (l3)[row + 1])
	val += 1;
      if (istk (l3)[row + m3] > istk (l3)[row + m3 + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not size array!\n");
      return 0;
    }
  wave_find (cstk (l4), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l4));
      return 0;
    }
  wave_fil_len_cal (cstk (l4), &filterLen);
  if ((istk (l3)[0] < filterLen) || (istk (l3)[m3] < filterLen))
    {
      sciprint ("The wavelet you select is not valid!\n");
      return 0;
    }
  if ((istk (l5)[0] < 1) || (istk (l5)[0] > (m3 - 1)))
    {
      sciprint ("Please input a reasonable level!\n");
      return 0;
    }

  pLen = malloc (m3 * n3 * sizeof (int));
  for (row = 0; row < n3; row++)
    {
      for (col = 0; col < m3; col++)
	pLen[row + col * n3] = istk (l3)[col + row * m3];
    }
  if (Rhs == 6)
    {
      GetRhsVar (6, "c", &m6, &n6, &l6);
      wextend_validate (cstk (l6), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m7 = pLen[(m3 - 1) * n3];
      n7 = pLen[(m3 - 1) * n3 + 1];
      CreateVar (7, "d", &m7, &n7, &l7);
      wrcoef2 (stk (l2), pLen, cstk (l4), (m3 - 1), istk (l5)[0],
	       cstk (l1), stk (l7), cstk (l6));
      LhsVar (1) = 7;
    }
  else
    {
      m6 = pLen[(m3 - 1) * n3];
      n6 = pLen[(m3 - 1) * n3 + 1];
      CreateVar (6, "d", &m6, &n6, &l6);
      wrcoef2 (stk (l2), pLen, cstk (l4), (m3 - 1), istk (l5)[0],
	       cstk (l1), stk (l6), c);
      LhsVar (1) = 6;
    }

  free (pLen);
  return 0;
}

int
intswtappcoef2 (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int l5, m5, n5, l6, m6, n6;
  static int minlhs = 1, maxlhs = 1, minrhs = 4, maxrhs = 5;
  int *pLen, row, col, find, ffind, val, size, filterLen;
  char c[] = { "symh" };

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "i", &m2, &n2, &l2);
  GetRhsVar (3, "c", &m3, &n3, &l3);
  GetRhsVar (4, "i", &m4, &n4, &l4);


  if ((m1 != 1) && (n1 != 1))
    {
      sciprint ("Inputs are not coefs!\n");
      return 0;
    }
  if ((m2 == 1) || (n2 == 1))
    {
      sciprint ("Inputs are not size arrays!\n");
      return 0;
    }
  size = 0;
  for (row = 0; row < (m2 - 1); row++)
    {
      size += (istk (l2)[row]) * (istk (l2)[row + m2]);
    }
  size = size * 3 + (istk (l2)[0]) * (istk (l2)[m2]);
  if (m1 * n1 != size)
    {
      sciprint ("Inputs are not size array and coefs!\n");
      return 0;
    }
  val = 0;
  for (row = 0; row < (m2 - 1); row++)
    {
      if (istk (l2)[row] > istk (l2)[row + 1])
	val += 1;
      if (istk (l2)[row + m2] > istk (l2)[row + m2 + 1])
	val += 1;
    }
  if (val != 0)
    {
      sciprint ("Inputs are not size array!\n");
      return 0;
    }
  wave_find (cstk (l3), &find);
  if (find == 0)
    {
      sciprint ("%s is not available!\n", cstk (l3));
      return 0;
    }
  wave_fil_len_cal (cstk (l3), &filterLen);
  if ((istk (l2)[0] < filterLen) || (istk (l2)[m2] < filterLen))
    {
      sciprint ("The wavelet you select is not valid!\n");
      return 0;
    }
  if ((istk (l4)[0] < 1) || (istk (l4)[0] > (m2 - 1)))
    {
      sciprint ("Please input a reasonable level!\n");
      return 0;
    }


  pLen = malloc (m2 * n2 * sizeof (int));
  for (row = 0; row < n2; row++)
    {
      for (col = 0; col < m2; col++)
	pLen[row + col * n2] = istk (l2)[col + row * m2];
    }

  if (Rhs == 5)
    {
      GetRhsVar (5, "c", &m5, &n5, &l5);
      wextend_validate (cstk (l5), &ffind);
      if (ffind == 0)
	{
	  sciprint ("Extend Method not available!\n");
	  return 0;
	}
      m6 = pLen[(m2 - 1 - istk (l4)[0]) * n2];
      n6 = pLen[(m2 - 1 - istk (l4)[0]) * n2 + 1];
      CreateVar (6, "d", &m6, &n6, &l6);
      appcoef2 (stk (l1), pLen, (m2 - 1), istk (l4)[0],
		cstk (l3), stk (l6), cstk (l5));
      LhsVar (1) = 6;
    }
  else
    {
      m5 = pLen[(m2 - 1 - istk (l4)[0]) * n2];
      n5 = pLen[(m2 - 1 - istk (l4)[0]) * n2 + 1];
      CreateVar (5, "d", &m5, &n5, &l5);
      appcoef2 (stk (l1), pLen, (m2 - 1), istk (l4)[0],
		cstk (l3), stk (l5), c);
      LhsVar (1) = 5;
    }

  free (pLen);
  return 0;
}

int
intswtcat (char *fname)
{
  static int l1, m1, n1, l2, m2, n2, l3, m3, n3, l4, m4, n4;
  static int minlhs = 1, maxlhs = 1, minrhs = 2, maxrhs = 3;
  double *me;
  int nt;

  CheckRhs (minrhs, maxrhs);
  CheckLhs (minlhs, maxlhs);

  GetRhsVar (1, "d", &m1, &n1, &l1);
  GetRhsVar (2, "d", &m2, &n2, &l2);

  /*if ((m1 != 1) && (n1 != 1))
     {
     sciprint("Please input vectors!\n");
     return 0;
     }
     if ((m2 != 1) && (n2 != 1))
     {
     sciprint("Please input vectors!\n");
     return 0;
     }
   */

  if (Rhs == 2)
    {
      n3 = n2 * m2 + n1 * m1;
      m3 = 1;
      CreateVar (3, "d", &m3, &n3, &l3);
      mem_copy (stk (l1), n1 * m1, stk (l2), n2 * m2, stk (l3));
      LhsVar (1) = 3;
    }
  else
    {
      GetRhsVar (3, "d", &m3, &n3, &l3);
      /*if ((m3 != 1) && (n3 != 1))
         {
         sciprint("Please input vectors!\n");
         return 0;
         }
       */
      nt = n2 * m2 + n1 * m1;
      me = malloc (nt * sizeof (double));
      mem_copy (stk (l1), n1 * m1, stk (l2), n2 * m2, me);
      n4 = nt + n3 * m3;
      m4 = 1;
      CreateVar (4, "d", &m4, &n4, &l4);
      mem_copy (me, nt, stk (l3), n3 * m3, stk (l4));
      LhsVar (1) = 4;
      free (me);
    }

  return 0;
}
