/*
 * -------------------------------------------------------------------------
 * swt.h -- Declarations for wavelet functions.
 * SWT - Scilab wavelet toolbox
 * Copyright (C) 2005-2006  Roger Liu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 */

#ifndef __SWT_H__
#define __SWT_H__ 1

extern void swtconv (int sigInLength, double *sigIn, int filterLength,
		     double *filter, int sigOutLength, double *sigOut);
extern void lenCal (int sigInLength, int *sigOutLength);
extern void downSamp2 (double *sigIn, int sigInLength, double *sigOut,
		       int sigOutLength);
extern void upSamp2 (double *sigIn, int sigInLength, double *sigOut);
extern void downSamp2r (double *sigIn, int sigInLength, double *sigOut,
			int sigOutLength);
extern void upSamp2r (double *sigIn, int sigInLength, double *sigOut);
extern void wrev (double *sigIn, int sigInLength, double *sigOut);
extern void qmf (double *sigIn, int sigInLength, double *sigOut);
extern void wextend (double *sigIn, int sigInLength, int addLength,
		     char *mode, double *sigOut);
extern void wextend_validate (char *mode, int *find);
extern void wkeep (double *sigIn, int sigInLength, double *sigOut,
		   int sigOutLength);
extern void filgroup (double *filterIn, int sigInLength, double *filterLowRec,
		      double *filterLowDec, double *filterHiRec,
		      double *filterHiDec);
extern void wave_find (char *waveType, int *find);
extern void wave_len_validate (int sigInLen, int waveLength, int *lev,
			       int *val);
extern void wave_fil_len_cal (char *waveType, int *waveLength);
extern void wave_fil_coef_dec (char *waveType, double *pLowDec,
			       double *pHiDec, int filterLen);
extern void wave_fil_coef_rec (char *waveType, double *pLowRec,
			       double *pHiRec, int filterLen);
extern void wave_fil_coef_extract (char *waveType, double *pLowRec,
				   int filterLen);
extern void dwt (double *sigIn, int sigInLength, char *waveType,
		 double *approx, double *detail, int filterLen,
		 int filterOutLength, int sigOutLength);
extern void dwtex (double *sigIn, int sigInLength, char *waveType,
		   double *approx, double *detail, int filterLen,
		   int filterOutLength, int sigOutLength, char *mode);
extern void idwt (char *waveType, double *approx, double *detail,
		  int sigInLength, int filterLen, int filterOutLength,
		  int sigOutLength, double *sigOut);
extern void idwtex (char *waveType, double *approx, double *detail,
		    int sigInLength, int filterLen, int filterOutLength,
		    int sigOutLength, double *sigOut, char *mode);
extern void wave_dec_len_cal (char *waveType, int sigLength, int stride,
			      int *waveDecLengthArray, int *total);
extern void wavedec (int sigInLength, double *sigIn, int sigOutLength,
		     double *sigOut, int stride, int *waveDecLengthArray,
		     char *waveType, char *mode);
extern void waverec (int sigInLength, double *sigIn, int sigOutLength,
		     double *sigOut, int stride, int *waveDecLengthArray,
		     char *waveType, char *mode);
extern void wrcoef (double *sigIn, int sigInLength, int stride,
		    char *waveType, double *sigOut, int sigOutLength,
		    char *coefType, int level, int *waveDecLengthArray,
		    char *mode);
extern void detcoef (int sigInLength, double *sigIn, int sigOutLength,
		     double *sigOut, int stride, int level,
		     int *waveDecLengthArray);
extern void appcoef (double *sigIn, int sigInLength, int stride,
		     char *waveType, double *sigOut, int sigOutLength,
		     int level, int *waveDecLengthArray, char *mode);
extern void matrix_tran (double *matrixIn, int matrixInRow, int matrixInCol,
			 double *matrixOut);
extern void swtdwt2D (double *matrixIn, int matrixInRow, int matrixInCol,
		      double *matrixOutApprox, double *matrixOutColDetail,
		      double *matrixOutRowDetail, double *matrixOutDetail,
		      char *waveType, char *mode);
extern void swtdwt2Dex (double *matrixIn, int matrixInRow, int matrixInCol,
			double *matrixOutApprox, double *matrixOutColDetail,
			double *matrixOutRowDetail, double *matrixOutDetail,
			char *waveType, char *mode);
extern void wkeep2 (double *matrixIn, int matrixInRow, int matrixInCol,
		    double *matrixOut, int matrixOutRow, int matrixOutCol);
extern void wextend2 (double *matrixIn, int matrixInRow, int matrixInCol,
		      int matrixOutRowAdded, int matrixOutColAdded,
		      char *mode, double *matrixOut);
extern void swtidwt2D (double *matrixInApprox, double *matrixInColDetail,
		       double *matrixInRowDetail, double *matrixInDetail,
		       int matrixInRow, int matrixInCol, double *matrixOut,
		       int matrixOutRow, int matrixOutCol, char *waveType,
		       char *mode);
extern void swtidwt2Dex (double *matrixInApprox, double *matrixInColDetail,
			 double *matrixInRowDetail, double *matrixInDetail,
			 int matrixInRow, int matrixInCol, double *matrixOut,
			 int matrixOutRow, int matrixOutCol, char *waveType,
			 char *mode);
extern void wavedec2 (double *matrixIn, int stride, char *waveType, int *pLen,
		      double *coef, char *mode);
extern void wave_mem_cal (int *pLen, int stride, int *total);
extern void matrix_locate (int stride, int *pLen, int *pH, int *pV, int *pD);
extern void matrix_wavedec_len_cal (int matrixInRow, int matrixInCol,
				    int stride, char *waveType, int *pLen);
extern void waverec2 (double *coef, int *pLen, int stride, char *waveType,
		      double *matrixOut, char *mode);
extern void detcoef2 (double *coef, int *pLen, int stride, int level,
		      char *coefType, double *coefOut);
extern void wrcoef2 (double *coef, int *pLen, char *waveType, int stride,
		     int level, char *type, double *matrixOut, char *mode);
extern void appcoef2 (double *coef, int *pLen, int stride, int level,
		      char *waveType, double *coefOut, char *mode);
extern void mem_copy (double *sigA, int lenA, double *sigB, int lenB,
		      double *sigOut);

#endif //__SWT_H__
