/*
 * -------------------------------------------------------------------------
 * swt2d.c -- functions for 2D wavelet.
 * SWT - Scilab wavelet toolbox
 * Copyright (C) 2005-2006  Roger Liu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 */

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "swt.h"

#if 0
void
row_dwt (double *matrixIn, int matrixInRow, int matrixInCol,
	 double *matrixOutApprox, double *matrixOutDetail,
	 int matrixOutRow, int matrixOutCol, char *waveType)
{

  int row, filterLen, filterOutLength;

  wave_fil_len_cal (waveType, &filterLen);
  filterOutLength = matrixInCol + filterLen - 1;
  for (row = 0; row < matrixInRow; row++)
    {
      dwt ((matrixIn + row * matrixInCol), matrixInCol,
	   waveType, (matrixOutApprox + row * matrixOutCol),
	   (matrixOutDetail + row * matrixOutCol), filterLen,
	   filterOutLength, matrixOutCol);
    }
  return;
}

void
swt_row_dwt (double *matrixIn, int matrixInRow, int matrixInCol,
	     double *matrixOutApprox, double *matrixOutDetail,
	     int matrixOutRow, int matrixOutCol, char *waveType)
{
  double *pMatrixInTemp, *pMatrixOutApprox, *pMatrixOutDetail;

  pMatrixInTemp = malloc (matrixInRow * matrixInCol * sizeof (double));
  pMatrixOutApprox = malloc (matrixOutRow * matrixOutCol * sizeof (double));
  pMatrixOutDetail = malloc (matrixOutRow * matrixOutCol * sizeof (double));

  matrix_tran (matrixIn, matrixInCol, matrixInRow, pMatrixInTemp);
  row_dwt (pMatrixInTemp, matrixInRow, matrixInCol, pMatrixOutApprox,
	   pMatrixOutDetail, matrixOutRow, matrixOutCol, waveType);
  matrix_tran (pMatrixOutApprox, matrixOutRow, matrixOutCol, matrixOutApprox);
  matrix_tran (pMatrixOutDetail, matrixOutRow, matrixOutCol, matrixOutDetail);
  free (pMatrixInTemp);
  free (pMatrixOutApprox);
  free (pMatrixOutDetail);
  return;
}
#endif /* 0 */

void
matrix_tran (double *matrixIn, int matrixInRow, int matrixInCol,
	     double *matrixOut)
{
  int row, col;

  for (col = 0; col < matrixInCol; col++)
    {
      for (row = 0; row < matrixInRow; row++)
	{
	  matrixOut[row + col * matrixInRow] =
	    matrixIn[col + row * matrixInCol];
	}
    }
  return;
}

void
swtdwt2D (double *matrixIn, int matrixInRow, int matrixInCol,
	  double *matrixOutApprox, double *matrixOutColDetail,
	  double *matrixOutRowDetail, double *matrixOutDetail, char *waveType,
	  char *mode)
{
  int row, col;
  int filterLen, filterOutLength, matrixOutRow, matrixOutCol;
  double temp;
  double *matrixInPre;
  double *pMatrixOutApproxPre, *pMatrixOutDetailPre;
  double *matrixOutApproxPre, *matrixOutDetailPre;

  /* Transpose */
  matrixInPre = malloc (matrixInRow * matrixInCol * sizeof (double));

  /* for(col=0;col<matrixInRow;col++)
     {
     for(row=0;row<matrixInCol;row++)
     {
     matrixInPre[row+col*matrixInCol]=
     matrixIn[col+row*matrixInRow];
     }
     }
   */
  matrix_tran (matrixIn, matrixInCol, matrixInRow, matrixInPre);

  /* Row DWT  */
  wave_fil_len_cal (waveType, &filterLen);
  filterOutLength = matrixInCol + filterLen - 1;
  lenCal (filterOutLength, &matrixOutCol);
  filterOutLength = matrixInRow + filterLen - 1;
  lenCal (filterOutLength, &matrixOutRow);

  matrixOutApproxPre = malloc (matrixInRow * matrixOutCol * sizeof (double));
  matrixOutDetailPre = malloc (matrixInRow * matrixOutCol * sizeof (double));
  filterOutLength = matrixInCol + filterLen - 1;
  for (row = 0; row < matrixInRow; row++)
    {
      dwtex ((matrixInPre + row * matrixInCol), matrixInCol,
	     waveType, (matrixOutApproxPre + row * matrixOutCol),
	     (matrixOutDetailPre + row * matrixOutCol), filterLen,
	     filterOutLength, matrixOutCol, mode);
    }

  free (matrixInPre);


  /* transpose */
  pMatrixOutApproxPre = malloc (matrixInRow * matrixOutCol * sizeof (double));
  matrix_tran (matrixOutApproxPre, matrixInRow,
	       matrixOutCol, pMatrixOutApproxPre);
  free (matrixOutApproxPre);

  pMatrixOutDetailPre = malloc (matrixInRow * matrixOutCol * sizeof (double));
  matrix_tran (matrixOutDetailPre, matrixInRow,
	       matrixOutCol, pMatrixOutDetailPre);
  free (matrixOutDetailPre);


  /* Col DWT */
  filterOutLength = matrixInRow + filterLen - 1;
  for (col = 0; col < matrixOutCol; col++)
    {
      dwtex ((pMatrixOutApproxPre + col * matrixInRow), matrixInRow,
	     waveType, (matrixOutApprox + col * matrixOutRow),
	     (matrixOutColDetail + col * matrixOutRow), filterLen,
	     filterOutLength, matrixOutRow, mode);
    }
  for (col = 0; col < matrixOutCol; col++)
    {
      dwtex ((pMatrixOutDetailPre + col * matrixInRow), matrixInRow,
	     waveType, (matrixOutRowDetail + col * matrixOutRow),
	     (matrixOutDetail + col * matrixOutRow), filterLen,
	     filterOutLength, matrixOutRow, mode);
    }

  free (pMatrixOutApproxPre);
  free (pMatrixOutDetailPre);

  return;
}

void
swtdwt2Dex (double *matrixIn, int matrixInRow, int matrixInCol,
	    double *matrixOutApprox, double *matrixOutColDetail,
	    double *matrixOutRowDetail, double *matrixOutDetail,
	    char *waveType, char *mode)
{
  int row, col, i, j;
  int filterLen, filterOutLength, matrixOutRow, matrixOutCol;
  int matrixOutRowM, matrixOutColM, matrixInRowM, matrixInColM;
  double temp;
  double *matrixInPre, *matrixInRo;
  double *pMatrixOutApproxPre, *pMatrixOutDetailPre;
  double *matrixOutApproxPre, *matrixOutDetailPre;
  double *matrixOutApproxM, *matrixOutColDetailM;
  double *matrixOutRowDetailM, *matrixOutDetailM;

  /* extension */
  wave_fil_len_cal (waveType, &filterLen);
  matrixInRowM = matrixInRow + 2 * filterLen;
  matrixInColM = matrixInCol + 2 * filterLen;
  matrixInRo = malloc (matrixInRowM * matrixInColM * sizeof (double));
  matrixInPre = malloc(matrixInRowM * matrixInColM * sizeof (double));
 
  wextend2 (matrixIn, matrixInRow, matrixInCol, filterLen,
	    filterLen, mode, matrixInRo);
  matrix_tran (matrixInRo, matrixInColM, matrixInRowM, matrixInPre);
  
  free (matrixInRo);

  /*for(i=0;i<matrixInRowM;i++)
    {
      for(j=0;j<matrixInColM;j++)
	printf("%d ",(int)(matrixInPre[i*matrixInColM+j]));
      printf("\n");

      }*/
  /* Transpose */

  /* matrix_tran(matrixInM, matrixInColM, matrixInRowM, matrixInPre);
     free(matrixInM);
   */


  /* Row DWT  */

  filterOutLength = matrixInColM + filterLen - 1;
  lenCal (filterOutLength, &matrixOutColM);
  filterOutLength = matrixInRowM + filterLen - 1;
  lenCal (filterOutLength, &matrixOutRowM);

  matrixOutApproxPre = malloc (matrixInRowM *
			       matrixOutColM * sizeof (double));
  matrixOutDetailPre = malloc (matrixInRowM *
			       matrixOutColM * sizeof (double));
  filterOutLength = matrixInColM + filterLen - 1;
  for (row = 0; row < matrixInRowM; row++)
    {
      dwt ((matrixInPre + row * matrixInColM), matrixInColM,
	   waveType, (matrixOutApproxPre + row * matrixOutColM),
	   (matrixOutDetailPre + row * matrixOutColM), filterLen,
	   filterOutLength, matrixOutColM);
    }

  free (matrixInPre);

  /* transpose */
  pMatrixOutApproxPre = malloc (matrixInRowM *
				matrixOutColM * sizeof (double));
  matrix_tran (matrixOutApproxPre, matrixInRowM,
	       matrixOutColM, pMatrixOutApproxPre);
  free (matrixOutApproxPre);

  pMatrixOutDetailPre = malloc (matrixInRowM *
				matrixOutColM * sizeof (double));
  matrix_tran (matrixOutDetailPre, matrixInRowM,
	       matrixOutColM, pMatrixOutDetailPre);
  free (matrixOutDetailPre);

  /* Col DWT */
  filterOutLength = matrixInRowM + filterLen - 1;
  matrixOutApproxM = malloc (matrixOutRowM * matrixOutColM * sizeof (double));
  matrixOutColDetailM = malloc (matrixOutRowM * matrixOutColM
				* sizeof (double));

  for (col = 0; col < matrixOutColM; col++)
    {
      dwt ((pMatrixOutApproxPre + col * matrixInRowM), matrixInRowM,
	   waveType, (matrixOutApproxM + col * matrixOutRowM),
	   (matrixOutColDetailM + col * matrixOutRowM), filterLen,
	   filterOutLength, matrixOutRowM);
    }
  free (pMatrixOutApproxPre);
  filterOutLength = matrixInCol + filterLen - 1;
  lenCal (filterOutLength, &matrixOutCol);
  filterOutLength = matrixInRow + filterLen - 1;
  lenCal (filterOutLength, &matrixOutRow);
  wkeep2 (matrixOutApproxM, matrixOutRowM, matrixOutColM, 
	  matrixOutApprox, matrixOutRow, matrixOutCol);
  free (matrixOutApproxM);
  wkeep2 (matrixOutColDetailM, matrixOutRowM, matrixOutColM,
	  matrixOutColDetail, matrixOutRow, matrixOutCol);
  free (matrixOutColDetailM);

  filterOutLength = matrixInRowM + filterLen - 1;
  matrixOutRowDetailM = malloc (matrixOutRowM * matrixOutColM
				* sizeof (double));
  matrixOutDetailM = malloc (matrixOutRowM * matrixOutColM * sizeof (double));
  for (col = 0; col < matrixOutColM; col++)
    {
      dwt ((pMatrixOutDetailPre + col * matrixInRowM), matrixInRowM,
	   waveType, (matrixOutRowDetailM + col * matrixOutRowM),
	   (matrixOutDetailM + col * matrixOutRowM), filterLen,
	   filterOutLength, matrixOutRowM);
    }
  free (pMatrixOutDetailPre);
  wkeep2 (matrixOutRowDetailM, matrixOutRowM, matrixOutColM,
	  matrixOutRowDetail, matrixOutRow, matrixOutCol);
  free (matrixOutRowDetailM);
  wkeep2 (matrixOutDetailM, matrixOutRowM, matrixOutColM,
	  matrixOutDetail, matrixOutRow, matrixOutCol);
  free (matrixOutDetailM);

  return;
}


void
wkeep2 (double *matrixIn, int matrixInRow, int matrixInCol, double *matrixOut,
	int matrixOutRow, int matrixOutCol)
{
  int row, col, startRow, startCol;
  double *matrixInTemp, *matrixOutTemp;

  matrixInTemp = malloc (matrixInRow * matrixInCol * sizeof (double));
  matrixOutTemp = malloc (matrixOutRow * matrixOutCol * sizeof (double));
  matrix_tran (matrixIn, matrixInCol, matrixInRow, matrixInTemp);
  if ((matrixInRow - matrixOutRow) % 2 != 0)
    {
      startRow = (matrixInRow - matrixOutRow) / 2 + 1;
    }
  else
    {
      startRow = (matrixInRow - matrixOutRow) / 2;
    }
  if ((matrixInCol - matrixOutCol) % 2 != 0)
    {
      startCol = (matrixInCol - matrixOutCol) / 2 + 1;
    }
  else
    {
      startCol = (matrixInCol - matrixOutCol) / 2;
    }

  for (row = startRow; row < startRow + matrixOutRow; row++)
    {
      for (col = startCol; col < startCol + matrixOutCol; col++)
	matrixOutTemp[col - startCol + (row - startRow) * matrixOutCol] =
	  matrixInTemp[col + row * matrixInCol];
    }
  matrix_tran (matrixOutTemp, matrixOutRow, matrixOutCol, matrixOut);
  free (matrixInTemp);
  free (matrixOutTemp);
  return;
}

void
wextend2 (double *matrixIn, int matrixInRow, int matrixInCol,
	  int matrixOutRowAdded, int matrixOutColAdded, char *mode,
	  double *matrixOut)
{
  int matrixOutRow, matrixOutCol, row, col;
  double *matrixInTemp, *matrixOutPre, *matrixOutTemp;

  matrixOutRow = matrixInRow + 2 * matrixOutRowAdded;
  matrixOutCol = matrixInCol + 2 * matrixOutColAdded;

  matrixInTemp = malloc (matrixInRow * matrixInCol * sizeof (double));
  matrix_tran (matrixIn, matrixInCol, matrixInRow, matrixInTemp);
  matrixOutPre = malloc (matrixInRow * matrixOutCol * sizeof (double));

  for (row = 0; row < matrixInRow; row++)
    wextend ((matrixInTemp + row * matrixInCol), matrixInCol,
	     matrixOutColAdded, mode, (matrixOutPre + row * matrixOutCol));
  free (matrixInTemp);

  matrixOutTemp = malloc (matrixInRow * matrixOutCol * sizeof (double));
  matrix_tran (matrixOutPre, matrixInRow, matrixOutCol, matrixOutTemp);
  free (matrixOutPre);

  for (col = 0; col < matrixOutCol; col++)
    wextend ((matrixOutTemp + col * matrixInRow), matrixInRow,
	     matrixOutRowAdded, mode, (matrixOut + col * matrixOutRow));
  free (matrixOutTemp);

  return;
}


void
swtidwt2D (double *matrixInApprox, double *matrixInColDetail,
	   double *matrixInRowDetail, double *matrixInDetail, int matrixInRow,
	   int matrixInCol, double *matrixOut, int matrixOutRow,
	   int matrixOutCol, char *waveType, char *mode)
{
  int row, col;
  int filterLen, filterOutLength;
  double *matrixOutApproxPre, *matrixOutDetailPre;
  double *matrixOutApproxTemp, *matrixOutDetailTemp;
  double *matrixOutPre;

  wave_fil_len_cal (waveType, &filterLen);
  filterOutLength = 2 * matrixInRow + filterLen - 1;

  /* Approx Calculation */
  matrixOutApproxPre = malloc (matrixOutRow * matrixInCol * sizeof (double));
  matrixOutApproxTemp = malloc (matrixOutRow * matrixInCol * sizeof (double));
  for (col = 0; col < matrixInCol; col++)
    idwtex (waveType, (matrixInApprox + col * matrixInRow),
	    (matrixInColDetail + col * matrixInRow), matrixInRow,
	    filterLen, filterOutLength, matrixOutRow,
	    (matrixOutApproxPre + col * matrixOutRow), mode);
  matrix_tran (matrixOutApproxPre, matrixInCol, matrixOutRow,
	       matrixOutApproxTemp);
  free (matrixOutApproxPre);

  /* Detail Calculation */
  matrixOutDetailPre = malloc (matrixOutRow * matrixInCol * sizeof (double));
  for (col = 0; col < matrixInCol; col++)
    idwtex (waveType, (matrixInRowDetail + col * matrixInRow),
	    (matrixInDetail + col * matrixInRow), matrixInRow,
	    filterLen, filterOutLength, matrixOutRow,
	    (matrixOutDetailPre + col * matrixOutRow), mode);
  matrixOutDetailTemp = malloc (matrixOutRow * matrixInCol * sizeof (double));
  matrix_tran (matrixOutDetailPre, matrixInCol, matrixOutRow,
	       matrixOutDetailTemp);
  free (matrixOutDetailPre);

  /* Final Inverse Transform */
  filterOutLength = 2 * matrixInCol + filterLen - 1;
  matrixOutPre = malloc (matrixOutRow * matrixOutCol * sizeof (double));
  for (row = 0; row < matrixOutRow; row++)
    idwtex (waveType, (matrixOutApproxTemp + row * matrixInCol),
	    (matrixOutDetailTemp + row * matrixInCol), matrixInCol,
	    filterLen, filterOutLength, matrixOutCol,
	    (matrixOutPre + row * matrixOutCol), mode);
  free (matrixOutApproxTemp);
  free (matrixOutDetailTemp);
  matrix_tran (matrixOutPre, matrixOutRow, matrixOutCol, matrixOut);
  free (matrixOutPre);

  return;
}

void
swtidwt2Dex (double *matrixInApprox, double *matrixInColDetail,
	     double *matrixInRowDetail, double *matrixInDetail,
	     int matrixInRow, int matrixInCol, double *matrixOut,
	     int matrixOutRow, int matrixOutCol, char *waveType, char *mode)
{
  int row, col;
  int filterLen, filterOutLength;
  int matrixInRowM, matrixInColM;
  double *matrixOutApproxPre, *matrixOutDetailPre;
  double *matrixOutApproxTemp, *matrixOutDetailTemp;
  double *matrixOutPre;
  double *matrixInApproxM, *matrixInColDetailM;
  double *matrixInRowDetailM, *matrixInDetailM;
  //double *matrixApproxRo, *matrixApproxRoM;
  //double *matrixColDetailRo, *matrixColDetailRoM;
  //double *matrixRowDetailRo, *matrixRowDetailRoM;
  //double *matrixDetailRo, *matrixDetailRoM;


  wave_fil_len_cal (waveType, &filterLen);
  matrixInRowM = matrixInRow + 2 * filterLen;
  matrixInColM = matrixInCol + 2 * filterLen;

  /* Approx extension */
  //matrixApproxRo = malloc (matrixInRow * matrixInCol * sizeof (double));
  //matrix_tran (matrixInApprox, matrixInCol, matrixInRow, matrixApproxRo);
  //matrixApproxRoM = malloc (matrixInRowM * matrixInColM * sizeof (double));
  matrixInApproxM = malloc (matrixInRowM * matrixInColM * sizeof (double));
  wextend2 (matrixInApprox, matrixInRow, matrixInCol,
	    filterLen, filterLen, mode, matrixInApproxM);
  //free (matrixApproxRo);
  
  //matrix_tran (matrixApproxRoM, matrixInRowM, matrixInColM, matrixInApproxM);
  //free (matrixApproxRoM);

  /* Col Detail Extension  */
  // matrixColDetailRo = malloc (matrixInRow * matrixInCol * sizeof (double));
  //matrix_tran (matrixInColDetail, matrixInCol, matrixInRow,
  //       matrixColDetailRo);
  //matrixColDetailRoM = malloc (matrixInRowM * matrixInColM * sizeof (double));

  matrixInColDetailM = malloc (matrixInRowM * matrixInColM * sizeof (double));
  wextend2 (matrixInColDetail, matrixInRow, matrixInCol,
	    filterLen, filterLen, mode, matrixInColDetailM);
//free (matrixColDetailRo);
  
//matrix_tran (matrixColDetailRoM, matrixInRowM, matrixInColM,
  //matrixInColDetailM);
//free (matrixColDetailRoM);

  /* Row Detail Extension */
  //matrixRowDetailRo = malloc (matrixInRow * matrixInCol * sizeof (double));
  //matrix_tran (matrixInRowDetail, matrixInCol, matrixInRow,
  //	       matrixRowDetailRo);
//matrixRowDetailRoM = malloc (matrixInRowM * matrixInColM * sizeof (double));
  matrixInRowDetailM = malloc (matrixInRowM * matrixInColM * sizeof (double));
  wextend2 (matrixInRowDetail, matrixInRow, matrixInCol,
	    filterLen, filterLen, mode, matrixInRowDetailM);
//free (matrixRowDetailRo);
  
//matrix_tran (matrixRowDetailRoM, matrixInRowM, matrixInColM,
  //matrixInRowDetailM);
//free (matrixRowDetailRoM);

  /* Detail Extension */
  //matrixDetailRo = malloc (matrixInRow * matrixInCol * sizeof (double));
  //matrix_tran (matrixInDetail, matrixInCol, matrixInRow, matrixDetailRo);
  //matrixDetailRoM = malloc (matrixInRowM * matrixInColM * sizeof (double));
  matrixInDetailM = malloc (matrixInRowM * matrixInColM * sizeof (double));

  wextend2 (matrixInDetail, matrixInRow, matrixInCol,
	    filterLen, filterLen, mode, matrixInDetailM);
  //free (matrixDetailRo);
  
  //matrix_tran (matrixDetailRoM, matrixInRowM, matrixInColM, matrixInDetailM);
  //free (matrixDetailRoM);

  filterOutLength = 2 * matrixInRowM + filterLen - 1;

  /* Approx Calculation */
  matrixOutApproxPre = malloc (matrixOutRow * matrixInColM * sizeof (double));
  matrixOutApproxTemp =
    malloc (matrixOutRow * matrixInColM * sizeof (double));
  for (col = 0; col < matrixInColM; col++)
    idwt (waveType, (matrixInApproxM + col * matrixInRowM),
	  (matrixInColDetailM + col * matrixInRowM), matrixInRowM,
	  filterLen, filterOutLength, matrixOutRow,
	  (matrixOutApproxPre + col * matrixOutRow));
  matrix_tran (matrixOutApproxPre, matrixInColM, matrixOutRow,
	       matrixOutApproxTemp);
  free (matrixOutApproxPre);
  free (matrixInApproxM);
  free (matrixInColDetailM);

  /* Detail Calculation */
  matrixOutDetailPre = malloc (matrixOutRow * matrixInColM * sizeof (double));
  for (col = 0; col < matrixInColM; col++)
    idwt (waveType, (matrixInRowDetailM + col * matrixInRowM),
	  (matrixInDetailM + col * matrixInRowM), matrixInRowM,
	  filterLen, filterOutLength, matrixOutRow,
	  (matrixOutDetailPre + col * matrixOutRow));
  matrixOutDetailTemp =
    malloc (matrixOutRow * matrixInColM * sizeof (double));
  matrix_tran (matrixOutDetailPre, matrixInColM, matrixOutRow,
	       matrixOutDetailTemp);
  free (matrixOutDetailPre);
  free (matrixInRowDetailM);
  free (matrixInDetailM);

  /* Final Inverse Transform */
  filterOutLength = 2 * matrixInColM + filterLen - 1;
  matrixOutPre = malloc (matrixOutRow * matrixOutCol * sizeof (double));
  for (row = 0; row < matrixOutRow; row++)
    idwt (waveType, (matrixOutApproxTemp + row * matrixInColM),
	  (matrixOutDetailTemp + row * matrixInColM), matrixInColM,
	  filterLen, filterOutLength, matrixOutCol,
	  (matrixOutPre + row * matrixOutCol));
  free (matrixOutApproxTemp);
  free (matrixOutDetailTemp);
  matrix_tran (matrixOutPre, matrixOutRow, matrixOutCol, matrixOut);
  free (matrixOutPre);

  return;
}


void
matrix_wavedec_len_cal (int matrixInRow, int matrixInCol, int stride,
			char *waveType, int *pLen)
{
  int row, col;
  int filterLen, filterOutLength;

  pLen[stride * 2] = matrixInRow;
  pLen[stride * 2 + 1] = matrixInCol;

  wave_fil_len_cal (waveType, &filterLen);
  for (row = (stride - 1); row >= 0; row--)
    {
      filterOutLength = pLen[(row + 1) * 2] + filterLen - 1;
      lenCal (filterOutLength, (pLen + row * 2));
      filterOutLength = pLen[(row + 1) * 2 + 1] + filterLen - 1;
      lenCal (filterOutLength, (pLen + row * 2 + 1));
    }

  return;
}

void
matrix_locate (int stride, int *pLen, int *pH, int *pV, int *pD)
{
  int count;
  int area, acre;

  /* *pH = (*pLen)*(*(pLen+1)); */
  pH[0] = pLen[0] * pLen[1];
  /* *pV = 2 * (*pH); */
  pV[0] = 2 * pH[0];
  /* *pD = 3 * (*pH); */
  pD[0] = 3 * pH[0];

  for (count = 1; count < stride; count++)
    {
      /* area = (*(pLen+(count-1)*2)) * (*(pLen+(count-1)*2+1)); */
      area = pLen[2 * (count - 1)] * pLen[2 * (count - 1) + 1];
      /* acre = (*(pLen+count*2)) * (*(pLen+count*2+1)); */
      acre = pLen[2 * count] * pLen[2 * count + 1];
      /* *(pH+count) = *(pH+count-1) + 2 * area; */
      pH[count] = pH[count - 1] + 3 * area;
      /* *(pV+count) = *(pV+count-1) + area + acre; */
      pV[count] = pV[count - 1] + 2 * area + acre;
      /* *(pD+count) = *(pD+count-1) + 2 * acre; */
      pD[count] = pD[count - 1] + area + 2 * acre;
    }
  /* *total = *(pD+stride-1)+ acre - 1; */

  return;
}

void
wave_mem_cal (int *pLen, int stride, int *total)
{
  int count = 0;
  (*total) = 4 * (pLen[0]) * (pLen[1]);
  for (count = 1; count < stride; count++)
    (*total) += 3 * (pLen[2 * count]) * (pLen[2 * count + 1]);
  return;
}

void
wavedec2 (double *matrixIn, int stride, char *waveType, int *pLen,
	  double *coef, char *mode)
{
  int count, row, col;
  int *pH, *pV, *pD;
  double *matrixInTemp;
  double *matrixOutApproxTemp;

  matrixInTemp = malloc ((pLen[stride * 2]) *
			 (pLen[stride * 2 + 1]) * sizeof (double));
  matrixOutApproxTemp = malloc ((pLen[(stride - 1) * 2]) *
				(pLen[(stride - 1) * 2 + 1]) *
				sizeof (double));
  pH = malloc (stride * sizeof (int));
  pV = malloc (stride * sizeof (int));
  pD = malloc (stride * sizeof (int));
  matrix_locate (stride, pLen, pH, pV, pD);

  for (row = 0; row < pLen[stride * 2]; row++)
    {
      for (col = 0; col < pLen[stride * 2 + 1]; col++)
	matrixInTemp[col + row * (pLen[stride * 2 + 1])] =
	  matrixIn[col + row * (pLen[stride * 2 + 1])];
    }

  for (count = stride - 1; count >= 0; count--)
    {
      swtdwt2Dex (matrixInTemp, pLen[2 * (count + 1)],
		  pLen[2 * (count + 1) + 1], matrixOutApproxTemp,
		  (pH[count] + coef), (pV[count] + coef),
		  (pD[count] + coef), waveType, mode);
      for (row = 0; row < pLen[2 * count]; row++)
	{
	  for (col = 0; col < pLen[2 * count + 1]; col++)
	    matrixInTemp[col + row * (pLen[2 * count + 1])] =
	      matrixOutApproxTemp[col + row * (pLen[2 * count + 1])];
	}
    }
  free (matrixInTemp);
  free (pH);
  free (pV);
  free (pD);
  for (row = 0; row < pLen[0]; row++)
    {
      for (col = 0; col < pLen[1]; col++)
	coef[col + row * (pLen[1])] =
	  matrixOutApproxTemp[col + row * (pLen[1])];
    }
  free (matrixOutApproxTemp);
  return;
}

void
waverec2 (double *coef, int *pLen, int stride, char *waveType,
	  double *matrixOut, char *mode)
{
  int count, row, col;
  int *pH, *pV, *pD;
  double *matrixOutTemp;
  double *matrixInApproxTemp;

  matrixOutTemp = malloc ((pLen[stride * 2]) *
			  (pLen[stride * 2 + 1]) * sizeof (double));
  matrixInApproxTemp = malloc ((pLen[stride * 2]) *
			       (pLen[stride * 2 + 1]) * sizeof (double));

  pH = malloc (stride * sizeof (int));
  pV = malloc (stride * sizeof (int));
  pD = malloc (stride * sizeof (int));
  matrix_locate (stride, pLen, pH, pV, pD);

  for (row = 0; row < pLen[0]; row++)
    {
      for (col = 0; col < pLen[1]; col++)
	matrixInApproxTemp[col + row * (pLen[1])] =
	  coef[col + row * (pLen[1])];
    }

  for (count = 0; count < stride; count++)
    {
      swtidwt2Dex (matrixInApproxTemp, (coef + pH[count]),
		   (coef + pV[count]), (coef + pD[count]),
		   pLen[count * 2], pLen[count * 2 + 1], matrixOutTemp,
		   pLen[(count + 1) * 2], pLen[(count + 1) * 2 + 1], waveType,
		   mode);
      for (row = 0; row < pLen[(count + 1) * 2]; row++)
	{
	  for (col = 0; col < pLen[(count + 1) * 2 + 1]; col++)
	    matrixInApproxTemp[col + row * (pLen[(count + 1) * 2 + 1])] =
	      matrixOutTemp[col + row * (pLen[(count + 1) * 2 + 1])];
	}
    }

  for (row = 0; row < pLen[stride * 2]; row++)
    {
      for (col = 0; col < pLen[stride * 2 + 1]; col++)
	matrixOut[col + row * (pLen[stride * 2 + 1])] =
	  matrixInApproxTemp[col + row * (pLen[stride * 2 + 1])];
    }


  free (pH);
  free (pV);
  free (pD);
  free (matrixInApproxTemp);
  free (matrixOutTemp);
  return;
}

void
detcoef2 (double *coef, int *pLen, int stride, int level, char *coefType,
	  double *coefOut)
{
  int row, col, sta;
  int *pH, *pV, *pD;

  pH = malloc (stride * sizeof (int));
  pV = malloc (stride * sizeof (int));
  pD = malloc (stride * sizeof (int));
  matrix_locate (stride, pLen, pH, pV, pD);
  if (strcmp (coefType, "h") == 0)
    {
      sta = pH[stride - level];
    }
  if (strcmp (coefType, "v") == 0)
    {
      sta = pV[stride - level];
    }
  if (strcmp (coefType, "d") == 0)
    {
      sta = pD[stride - level];
    }

  for (row = 0; row < pLen[(stride - level) * 2]; row++)
    {
      for (col = 0; col < pLen[(stride - level) * 2 + 1]; col++)
	coefOut[col + row * (pLen[(stride - level) * 2 + 1])] =
	  coef[sta + col + row * (pLen[(stride - level) * 2 + 1])];
    }
  free (pH);
  free (pV);
  free (pD);

  return;
}

void
wrcoef2 (double *coef, int *pLen, char *waveType, int stride, int level,
	 char *type, double *matrixOut, char *mode)
{
  int count, total, sta, si;
  double *coefTemp;
  int *pH, *pV, *pD;

  wave_mem_cal (pLen, stride, &total);
  coefTemp = malloc (total * sizeof (double));

  pH = malloc (stride * sizeof (int));
  pV = malloc (stride * sizeof (int));
  pD = malloc (stride * sizeof (int));
  matrix_locate (stride, pLen, pH, pV, pD);

  for (count = 0; count < total; count++)
    coefTemp[count] = 0;

  if (strcmp (type, "h") == 0)
    {
      sta = pH[stride - level];
      si = (pLen[(stride - level) * 2]) * (pLen[(stride - level) * 2 + 1]);
    }
  if (strcmp (type, "v") == 0)
    {
      sta = pV[stride - level];
      si = (pLen[(stride - level) * 2]) * (pLen[(stride - level) * 2 + 1]);
    }
  if (strcmp (type, "d") == 0)
    {
      sta = pD[stride - level];
      si = (pLen[(stride - level) * 2]) * (pLen[(stride - level) * 2 + 1]);
    }
  if (strcmp (type, "a") == 0)
    {
      sta = 0;
      si = (pLen[0]) * (pLen[1]);
      if (level != stride)
	{
	  for (count = 1; count <= (stride - level); count++)
	    si += 3 * (pLen[(count - 1) * 2]) * (pLen[(count - 1) * 2 + 1]);
	}
    }

  for (count = sta; count < (si + sta); count++)
    coefTemp[count] = coef[count];

  waverec2 (coefTemp, pLen, stride, waveType, matrixOut, mode);

  free (pH);
  free (pV);
  free (pD);
  free (coefTemp);
  return;
}

void
appcoef2 (double *coef, int *pLen, int stride, int level, char *waveType,
	  double *coefOut, char *mode)
{
  int count;

  if (level == stride)
    {
      for (count = 0; count < ((pLen[0]) * (pLen[1])); count++)
	coefOut[count] = coef[count];
    }
  else
    {
      waverec2 (coef, pLen, (stride - level), waveType, coefOut, mode);
    }

  return;
}
